
local twimer = require'twimer'()

local hero_width = 50
local hero_height = 50

local heroes

local show_initial_banner = true
local show_final_banner = false

local intro_tween_time = 1
local intro_pause_time = 0.5
local intro_appearance_order
local intro_heroes = {}

love.graphics.setFont(love.graphics.newFont(40))

local function intro_enter_next()

  -- Calculate N (the new number of heroes)
  local N = #intro_heroes + 1

  if N > #intro_appearance_order then
    -- End of tween
    show_final_banner = true
    return
  end

  -- Give time for the tween to complete, and then some more
  twimer:after(intro_tween_time + intro_pause_time, intro_enter_next)

  -- Calculate the position where to insert the next character
  local insert_point = math.floor(N/2) + 1

  -- Insert the new hero at insert_point
  table.insert(intro_heroes, insert_point, { hero = intro_appearance_order[N] })

  local padding = math.max(math.floor(hero_width / (N - 0.5) + 5), 5)

  local size_c2c = (N - 1) * hero_width + (N - 1) * padding
  local left_base = -math.floor(size_c2c / 2)

  for i = 1, N do
    local new_x = left_base + (i - 1) * (hero_width + padding)
    local hero = intro_heroes[i]
    if i == insert_point then
      -- Set the new character old x = new x, and old_y out of the screen
      hero.old_x = new_x
      hero.old_y = (love.graphics.getHeight() + hero_height / 2)
    else
      hero.old_x = hero.new_x
      hero.old_y = hero.new_y
    end
    hero.new_x = new_x
    hero.new_y = math.floor(love.graphics.getHeight() / 2)
    hero.timer = twimer:manual_tween(intro_tween_time) -- restart tween
  end
end

function love.load(args)
  heroes = {
    ichi = { img = love.graphics.newImage('ichi.png') },
    ni = { img = love.graphics.newImage('ni.png') },
    san = { img = love.graphics.newImage('san.png') },
    shi = { img = love.graphics.newImage('shi.png') }
  }

  intro_appearance_order = { heroes.ichi, heroes.ni, heroes.san, heroes.shi }

  twimer:after(3, function()
    show_initial_banner = false
    intro_heroes = {}
    intro_enter_next()
  end)
end

function love.update(dt)
  twimer:update(dt)
end

function love.draw()
  love.graphics.setColor(1, 1, 1)
  if show_initial_banner then
    local txt = "Here come our heroes!"
    local font = love.graphics.getFont()
    love.graphics.print(txt,
      math.floor((love.graphics.getWidth() - love.graphics.getFont():getWidth(txt)) * 0.5),
      math.floor((love.graphics.getHeight() - font:getHeight()) * 0.5)
    )
  end

  -- Draw the heroes
  for i = 1, #intro_heroes do
    local hero = intro_heroes[i]
    love.graphics.draw(hero.hero.img,
      love.graphics.getWidth() * 0.5 +
      math.floor(twimer.lerp(hero.old_x, hero.new_x, hero.timer.val) + 0.5),
      math.floor(twimer.lerp(hero.old_y, hero.new_y, hero.timer.val) + 0.5),
      0, 1, 1,
      hero_width / 2, hero_height / 2
    )
  end

  if show_final_banner then
    love.graphics.setColor(1,0,0)
    local txt = "The battle begins!"
    local font = love.graphics.getFont()
    love.graphics.print(txt,
      math.floor((love.graphics.getWidth() - love.graphics.getFont():getWidth(txt)) * 0.5),
      math.floor((love.graphics.getHeight() - font:getHeight() + hero_height) * 0.5) + 30
    )
    love.graphics.setColor(1, 1, 1)
  end
end


function love.keypressed(k) return k == "escape" and love.event.quit() end
